@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2019 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2020 Savoir-faire Linux Inc.
  *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,15 @@
 
 namespace dht {
 
+namespace net {
+    class DatagramSocket;
+}
+
 class OPENDHT_PUBLIC DhtInterface {
 public:
     DhtInterface() = default;
+    DhtInterface(const Logger& l) : logger_(std::make_shared<Logger>(l)) {};
+    DhtInterface(const std::shared_ptr<Logger>& l) : logger_(l) {};
     virtual ~DhtInterface() = default;
 
     // [[deprecated]]
@@ -36,9 +42,12 @@ public:
     /**
      * Get the current status of the node for the given family.
      */
+    virtual NodeStatus updateStatus(sa_family_t af) { return getStatus(af); };
     virtual NodeStatus getStatus(sa_family_t af) const = 0;
     virtual NodeStatus getStatus() const = 0;
 
+    virtual net::DatagramSocket* getSocket() const { return {}; };
+
     /**
      * Get the ID of the DHT node.
      */
@@ -61,19 +70,21 @@ public:
 
     virtual const ValueType& getType(ValueType::Id type_id) const = 0;
 
+    virtual void addBootstrap(const std::string& /*host*/, const std::string& /*service*/) {};
+    virtual void clearBootstrap() {};
+
     /**
      * Insert a node in the main routing table.
      * The node is not pinged, so this should be
      * used to bootstrap efficiently from previously known nodes.
      */
     virtual void insertNode(const InfoHash& id, const SockAddr&) = 0;
-    virtual void insertNode(const InfoHash& id, const sockaddr* sa, socklen_t salen) = 0;
     virtual void insertNode(const NodeExport& n) = 0;
 
-    virtual void pingNode(const sockaddr*, socklen_t, DoneCallbackSimple&& cb={}) = 0;
+    virtual void pingNode(SockAddr, DoneCallbackSimple&& cb={}) = 0;
 
-    virtual time_point periodic(const uint8_t *buf, size_t buflen, const SockAddr&) = 0;
-    virtual time_point periodic(const uint8_t *buf, size_t buflen, const sockaddr* from, socklen_t fromlen) = 0;
+    virtual time_point periodic(const uint8_t *buf, size_t buflen, SockAddr, const time_point& now) = 0;
+    virtual time_point periodic(const uint8_t *buf, size_t buflen, const sockaddr* from, socklen_t fromlen, const time_point& now) = 0;
 
     /**
      * Get a value by searching on all available protocols (IPv4, IPv6),
@@ -214,11 +225,24 @@ public:
     /**
      * Enable or disable logging of DHT internal messages
      */
-    virtual void setLoggers(LogMethod error = NOLOG, LogMethod warn = NOLOG, LogMethod debug = NOLOG)
-    {
-        DHT_LOG.DBG = debug;
-        DHT_LOG.WARN = warn;
-        DHT_LOG.ERR = error;
+    virtual void setLoggers(LogMethod error = {}, LogMethod warn = {}, LogMethod debug = {}) {
+        if (logger_) {
+            logger_->DBG = std::move(debug);
+            logger_->WARN = std::move(warn);
+            logger_->ERR = std::move(error);
+        } else
+            logger_= std::make_shared<Logger>(std::move(error), std::move(warn), std::move(debug));
+    }
+
+    virtual void setLogger(const Logger& l) {
+        if (logger_)
+            *logger_ = l;
+        else
+            logger_= std::make_shared<Logger>(l);
+    }
+
+    virtual void setLogger(const std::shared_ptr<Logger>& l) {
+        logger_ = l;
     }
 
     /**
@@ -226,7 +250,8 @@ public:
      */
     virtual void setLogFilter(const InfoHash& f)
     {
-        DHT_LOG.setFilter(f);
+        if (logger_)
+            logger_->setFilter(f);
     }
 
     virtual void setPushNotificationToken(const std::string&) {};
@@ -238,9 +263,7 @@ public:
     virtual void pushNotificationReceived(const std::map<std::string, std::string>& data) = 0;
 
 protected:
-    bool logFilerEnable_ {};
-    InfoHash logFiler_ {};
-    Logger DHT_LOG;
+    std::shared_ptr<Logger> logger_ {};
 };
 
 } // namespace dht
